package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAppIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.back.base.tencent.pojo.dto.TencentOrderDTO;
import com.zanxiang.game.back.base.tencent.pojo.dto.TencentUserDTO;
import com.zanxiang.game.back.serve.dao.mapper.GameBackPolicyMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameBackPolicyDTO;
import com.zanxiang.game.back.serve.pojo.dto.GameBackPolicyListDTO;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameBackPolicyVO;
import com.zanxiang.game.back.serve.service.IGameBackPolicyService;
import com.zanxiang.game.back.serve.service.IGameTencentBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentOrderService;
import com.zanxiang.game.back.serve.service.IGameTencentUserService;
import com.zanxiang.game.back.serve.utils.BackPolicyUtil;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.NumberUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 游戏回传策略表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Slf4j
@Service
public class GameBackPolicyServiceImpl extends ServiceImpl<GameBackPolicyMapper, GameBackPolicy> implements IGameBackPolicyService {
    @Autowired
    private IGameTencentBackLogService gameTencentBackLogService;
    @Autowired
    private IGameTencentOrderService gameTencentOrderService;
    @Autowired
    private IGameTencentUserService gameTencentUserService;
    @Autowired
    private IGameBackPolicyService gameBackPolicyService;
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    /**
     * 支付状态，0 : 预下单, 1 : 待支付，2 : 支付成功，-1 : 已取消
     */
    private static final Integer COMPLETE_ORDER = 0;
    private static final Integer PURCHASE = 2;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean tencentOrderBack(TencentOrderDTO dto) {
        log.error("回传腾讯订单开始，dto：{}", JsonUtil.toString(dto));
        //订单保存
        GameTencentOrder gameTencentOrder = GameTencentOrder.builder()
                .orderId(dto.getOrderId())
                .channel(dto.getChannel())
                .rechargeMoney(dto.getRechargeMoney())
                .rechargeTime(dto.getRechargeTime())
                .subscribeTime(dto.getSubscribeTime())
                .registerTime(dto.getRegisterTime())
                .wechatAppId(dto.getWechatAppId())
                .wechatOpenid(dto.getWechatOpenid())
                .orderStatus(dto.getOrderStatus())
                .payTime(dto.getPayTime())
                .isBack(BackStatusEnum.NO.getBackStatus())
                .backPolicyId(dto.getBackPolicyId())
                .build();
        gameTencentOrderService.save(gameTencentOrder);
        if (!dto.getOrderStatus().equals(COMPLETE_ORDER) && !dto.getOrderStatus().equals(PURCHASE)) {
            return true;
        }

        String actionType = gameTencentOrder.getOrderStatus().equals(PURCHASE) ?
                ActionTypeEnum.PURCHASE.getActionType() : ActionTypeEnum.COMPLETE_ORDER.getActionType();
        DataReportOfAppIdRpcDTO dataReportOfAppIdRpcDTO = DataReportOfAppIdRpcDTO.builder()
                .appId(dto.getWechatAppId())
                .action(UserActionRpcDTO.builder()
                        .actionTime(dto.getOrderStatus().equals(PURCHASE) ? dto.getPayTime() : dto.getRechargeTime())
                        .actionType(actionType)
                        .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                .wechatAppId(dto.getWechatAppId())
                                .wechatOpenid(dto.getWechatOpenid())
                                .build())
                        .build())
                .build();

        GameTencentBackLog gameTencentBackLog = GameTencentBackLog.builder()
                .gameId(dto.getGameId())
                .adAccountId(dto.getAdAccountId())
                .actionTime(dto.getOrderStatus().equals(PURCHASE) ? dto.getPayTime() : dto.getRechargeTime())
                .createTime(LocalDateTime.now())
                .actionType(actionType)
                .orderId(dto.getOrderId())
                .wechatAppId(dto.getWechatAppId())
                .wechatOpenid(dto.getWechatOpenid())
                .build();

        boolean doBack;
        if (null == dto.getBackPolicyId()) {
            doBack = true;
        } else {
            GameBackPolicy gameBackPolicy = gameBackPolicyService.getById(dto.getBackPolicyId());

            doBack = BackPolicyUtil.backOrder(gameBackPolicy, dto.getRechargeMoney(),
                    () -> gameTencentOrderService.count(new LambdaQueryWrapper<GameTencentOrder>()
                            .eq(GameTencentOrder::getGameId, dto.getGameId())
                            .eq(GameTencentOrder::getWechatAppId, dto.getWechatAppId())
                            .eq(GameTencentOrder::getAdAccountId, dto.getAdAccountId())
                            .eq(GameTencentOrder::getOrderStatus, 2)
                            .eq(GameTencentOrder::getWechatOpenid, dto.getWechatOpenid())
                            .last("limit 1")
                    ) < 1, backUnit -> Long.valueOf(gameTencentOrderService.count(new LambdaQueryWrapper<GameTencentOrder>()
                            .eq(GameTencentOrder::getGameId, dto.getGameId())
                            .eq(GameTencentOrder::getWechatAppId, dto.getWechatAppId())
                            .eq(GameTencentOrder::getAdAccountId, dto.getAdAccountId())
                            .eq(GameTencentOrder::getOrderStatus, 2)
                            .last("limit " + backUnit)
                            .orderByDesc(GameTencentOrder::getCreateTime)
                    )).intValue());
        }

        try {
            if (dto.getOrderStatus().equals(COMPLETE_ORDER) || doBack) {
                userActionSetRpc.reportByAppId(dataReportOfAppIdRpcDTO);
                gameTencentOrder.setIsBack(BackStatusEnum.FAILED.getBackStatus());
                gameTencentOrderService.updateById(gameTencentOrder);

                gameTencentBackLogService.save(gameTencentBackLog);
            }
        } catch (Exception e) {
            log.error("回传腾讯订单失败，失败原因：{}", e.getMessage());
            gameTencentOrder.setBackLog(e.getMessage());
            gameTencentOrder.setIsBack(BackStatusEnum.FAILED.getBackStatus());
            gameTencentOrderService.updateById(gameTencentOrder);

            gameTencentBackLog.setBackLog(e.getMessage());
            gameTencentBackLogService.saveOrUpdate(gameTencentBackLog);
            return false;
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean tencentUserBack(TencentUserDTO dto) {
        log.error("回传腾讯用户开始，dto：{}", JsonUtil.toString(dto));
        GameTencentUser gameTencentUser = gameTencentUserService.getOne(new LambdaQueryWrapper<GameTencentUser>()
                .eq(GameTencentUser::getWechatOpenid, dto.getWechatOpenid())
                .eq(GameTencentUser::getGameId, dto.getGameId())
                .eq(GameTencentUser::getWechatAppId, dto.getWechatAppId())
                .eq(GameTencentUser::getAdAccountId, dto.getAdAccountId())
                .eq(GameTencentUser::getIsBack, BackStatusEnum.SUCCESS.getBackStatus())
        );
        if (null == gameTencentUser) {
            //用户保存
            gameTencentUser = GameTencentUser.builder()
                    .adAccountId(dto.getAdAccountId())
                    .gameId(dto.getGameId())
                    .channel(dto.getChannel())
                    .subscribeTime(dto.getSubscribeTime())
                    .registerTime(dto.getRegisterTime())
                    .wechatAppId(dto.getWechatAppId())
                    .wechatOpenid(dto.getWechatOpenid())
                    .isBack(BackStatusEnum.NO.getBackStatus())
                    .build();
            gameTencentUserService.save(gameTencentUser);
        }

        DataReportOfAppIdRpcDTO dataReportOfAppIdRpcDTO = DataReportOfAppIdRpcDTO.builder()
                .appId(dto.getWechatAppId())
                .action(UserActionRpcDTO.builder()
                        .actionTime(dto.getRegisterTime())
                        .actionType(ActionTypeEnum.REGISTER.getActionType())
                        .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                .wechatAppId(dto.getWechatAppId())
                                .wechatOpenid(dto.getWechatOpenid())
                                .build())
                        .build())
                .build();

        GameTencentBackLog gameTencentBackLog = GameTencentBackLog.builder()
                .gameId(dto.getGameId())
                .adAccountId(dto.getAdAccountId())
                .actionTime(dto.getRegisterTime())
                .createTime(LocalDateTime.now())
                .actionType(ActionTypeEnum.REGISTER.getActionType())
                .wechatAppId(dto.getWechatAppId())
                .wechatOpenid(dto.getWechatOpenid())
                .build();
        try {
            userActionSetRpc.reportByAppId(dataReportOfAppIdRpcDTO);
            gameTencentUser.setIsBack(BackStatusEnum.SUCCESS.getBackStatus());
            gameTencentUserService.updateById(gameTencentUser);

            gameTencentBackLogService.save(gameTencentBackLog);
        } catch (Exception e) {
            log.error("回传腾讯用户失败，失败原因：{}", e.getMessage());
            gameTencentUser.setBackLog(e.getMessage());
            gameTencentUser.setIsBack(BackStatusEnum.FAILED.getBackStatus());
            gameTencentUserService.updateById(gameTencentUser);

            gameTencentBackLog.setBackLog(e.getMessage());
            gameTencentBackLogService.saveOrUpdate(gameTencentBackLog);
            return false;
        }
        return true;

    }

    @Override
    public IPage<GameBackPolicyVO> policyList(GameBackPolicyListDTO dto) {
        return gameBackPolicyService.page(dto.toPage(), new LambdaQueryWrapper<GameBackPolicy>()
                        .like(StringUtils.isNotBlank(dto.getBackPolicyName()), GameBackPolicy::getBackPolicyName, dto.getBackPolicyName())
                        .eq(GameBackPolicy::getCreateBy, SecurityUtil.getUserId())
                        .orderByDesc(GameBackPolicy::getCreateTime))
                .convert(this::toVO);
    }

    @Override
    public List<GameBackPolicyVO> allOfUser() {
        return list(new LambdaQueryWrapper<GameBackPolicy>()
                .eq(GameBackPolicy::getCreateBy, SecurityUtil.getUserId())
                .orderByDesc(GameBackPolicy::getCreateTime)
        ).stream().map(this::toVOSimple).collect(Collectors.toList());
    }

    @Override
    public boolean createPolicy(GameBackPolicyDTO dto) {
        if (nameExist(null, SecurityUtil.getUserId(), dto.getBackPolicyName())) {
            throw new BaseException("回传策略名重复");
        }
        GameBackPolicy backPolicy = GameBackPolicy.builder()
                .backPolicyName(dto.getBackPolicyName())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMinMoneyType(dto.getFirstMinMoneyType())
                .firstMinMoneyRate(dto.getFirstMinMoneyRate())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMaxMoneyType(dto.getFirstMaxMoneyType())
                .firstMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .firstOtherMoneyType(dto.getFirstOtherMoneyType())
                .firstOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .rechargeMinMoneyType(dto.getFirstMinMoneyType())
                .rechargeMinMoneyRate(dto.getFirstMinMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .rechargeMaxMoneyType(dto.getFirstMaxMoneyType())
                .rechargeMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .rechargeOtherMoneyType(dto.getFirstOtherMoneyType())
                .rechargeOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .levelDown(dto.getLevelDown())
                .createBy(SecurityUtil.getUserId())
                .createTime(LocalDateTime.now())
                .build();
        return gameBackPolicyService.save(backPolicy);
    }

    @Override
    public boolean updatePolicy(Long policyId, GameBackPolicyDTO dto) {
        if (nameExist(policyId, SecurityUtil.getUserId(), dto.getBackPolicyName())) {
            throw new BaseException("回传策略名重复");
        }
        GameBackPolicy old = getById(policyId);
        GameBackPolicy backPolicy = GameBackPolicy.builder()
                .backPolicyName(dto.getBackPolicyName())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMinMoneyType(dto.getFirstMinMoneyType())
                .firstMinMoneyRate(dto.getFirstMinMoneyRate())
                .firstMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .firstMaxMoneyType(dto.getFirstMaxMoneyType())
                .firstMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .firstOtherMoneyType(dto.getFirstOtherMoneyType())
                .firstOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .rechargeMinMoneyType(dto.getFirstMinMoneyType())
                .rechargeMinMoneyRate(dto.getFirstMinMoneyRate())
                .rechargeMaxMoney(NumberUtil.multiply100(dto.getFirstMaxMoney()).longValue())
                .rechargeMaxMoneyType(dto.getFirstMaxMoneyType())
                .rechargeMaxMoneyRate(dto.getFirstMaxMoneyRate())
                .rechargeOtherMoneyType(dto.getFirstOtherMoneyType())
                .rechargeOtherMoneyRate(dto.getFirstOtherMoneyRate())
                .levelDown(dto.getLevelDown())
                .createBy(old.getCreateBy())
                .createTime(old.getCreateTime())
                .updateBy(SecurityUtil.getUserId())
                .updateTime(LocalDateTime.now())
                .build();

        return gameBackPolicyService.updateById(backPolicy);
    }

    @Override
    public boolean deletePolicy(List<Long> ids) {
        return gameBackPolicyService.removeByIds(ids);
    }

    private boolean nameExist(Long policyId, Long userId, String policyName) {
        return count(new LambdaQueryWrapper<GameBackPolicy>()
                .ne(policyId != null, GameBackPolicy::getId, policyId)
                .eq(GameBackPolicy::getBackPolicyName, policyName)
                .eq(GameBackPolicy::getCreateBy, userId)
        ) > 0;
    }

    private GameBackPolicyVO toVOSimple(GameBackPolicy gameBackPolicy) {
        if (gameBackPolicy == null) {
            return null;
        }
        GameBackPolicyVO vo = BeanUtil.copy(gameBackPolicy, GameBackPolicyVO.class);
        vo.setFirstMinMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getFirstMinMoney())));
        vo.setFirstMaxMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getFirstMaxMoney())));
        vo.setRechargeMinMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getRechargeMinMoney())));
        vo.setRechargeMaxMoney(NumberUtil.divide100(new BigDecimal(gameBackPolicy.getRechargeMaxMoney())));
        return vo;
    }

    private GameBackPolicyVO toVO(GameBackPolicy gameBackPolicy) {
        if (gameBackPolicy == null) {
            return null;
        }
        GameBackPolicyVO vo = toVOSimple(gameBackPolicy);
        vo.setCreateName(sysUserRpc.getById(gameBackPolicy.getCreateBy()).getData().getNickname());
        return vo;
    }
}
